# -*- coding: utf-8 -*-

import socket
import time
from ..log import logger
from .. import constants


class Client(object):
    """
    连接
    """

    host = None
    port = None
    sock = None

    def __init__(self, host, port):
        self.host = host
        self.port = port

    def write(self, data):
        """
        发送
        :param data:
        :return:
        """

        while True:
            # 先要保证连接可用
            self._make_sure_connected()

            if self._write_to_fd(data):
                # 写入成功
                return True

    def close(self):
        """
        关闭连接
        :return:
        """
        if self.sock:
            try:
                self.sock.close()
            except:
                pass

            self.sock = None

    def closed(self):
        """
        检查连接
        :return:
        """

        return not self.sock

    def _write_to_fd(self, data):
        """
        写入
        """

        while data:
            try:
                num_bytes = self.sock.send(data)
            except KeyboardInterrupt, e:
                raise e
            except Exception, e:
                logger.error('socket send fail. e: %s, data: %r', e, data[:100])
                # 关闭连接
                self.close()
                return False

            data = data[num_bytes:]

        return True

    def _make_sure_connected(self):
        """
        在写入之前调用，保证连接可用
        :return:
        """
        while self.closed():

            while True:
                self.sock = socket.socket()
                try:
                    self.sock.connect((self.host, self.port))
                except KeyboardInterrupt, e:
                    raise e
                except Exception, e:
                    logger.error('connect fail, host: %s, port: %s, sleep %ss, e: %s',
                                 self.host, self.port, constants.TRY_CONNECT_INTERVAL, e)
                    self.close()
                    time.sleep(constants.TRY_CONNECT_INTERVAL)
                else:
                    # 连接成功
                    break
