# -*- coding: utf-8 -*-

from ..gw_box import GWBox
from ..gw_pb2 import RspToUsers, CloseUsers
from .client import Client
from .. import constants


class Trigger(object):

    box_class = None
    client = None

    def __init__(self, box_class, host, port):
        self.box_class = box_class
        self.client = Client(host, port)

    def start(self):
        """
        废弃，无需调用
        :return:
        """
        pass

    def write_to_worker(self, data):
        """
        透传到worker进行处理
        """

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_TRIGGER_REQ

        if isinstance(data, self.box_class):
            # 打包
            data = data.pack()
        elif isinstance(data, dict):
            data = self.box_class(data).pack()
            
        gw_box.body = data

        msg = gw_box.pack()

        return self._write(msg)

    def write_to_users(self, data_list):
        """
        格式为
        [(uids, box), ...]
        :param data_list:
        :return:
        """

        msg = RspToUsers()

        for uids, data in data_list:
            if isinstance(data, self.box_class):
                data = data.pack()
            elif isinstance(data, dict):
                data = self.box_class(data).pack()

            row = msg.rows.add()
            row.buf = data
            row.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_WRITE_TO_USERS
        gw_box.body = msg.SerializeToString()

        msg = gw_box.pack()

        return self._write(msg)

    def close_users(self, uids):
        msg = CloseUsers()
        msg.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_CLOSE_USERS
        gw_box.body = msg.SerializeToString()

        msg = gw_box.pack()

        return self._write(msg)

    def _write(self, msg):
        return self.client.write(msg)
